%Draw base station (BS) and user locations.
%[BS_location, user_loc_cell, dist_user_BS_cell] = 
%BSUserLocation(N, U, cell_radius, exclude_disk_radius), where:
%
%N                   : number of BS
%U                   : number of users per cell
%cell_radius         : the cell (circum)radius [m].
%exclude_disk_radius : radius [m] of a disc around the BS where no users
%are allowed.
%
%Users are drawn uniformly.
function [BS_location, user_loc_cell, dist_user_BS_cell] = BSUserLocation(N, U, cell_radius, exclude_disk_radius)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% User locations.                                     %
% user_loc : N x U x 2 array                          %
% N - cell index. U - number of users in each cell.   %
% 2-> x,y coord.                                      %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
user_loc_cell = zeros(N,U,2);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Distance from user u in cell n to BS in cell "all" n.  %
% dist_user_BS_cell : N x U x N array                    %
% N - cell index. U - number of users in each cell.      %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
dist_user_BS_cell = zeros(N,U,N);

%Locate N cells, coordinate axes with 0,0 in center of cell one.
%              ___
%          ___/ 15\___
%      ___/ 14\___/ 16\__       Coordinates of hexagonal cells
%     / 13\___/ 4 \___/ 17\     numbered from 1 to 19. Center of cell 1
%     \___/ 5 \___/ 3 \___/     is (0,0)
%     / 12\___/ 1 \___/ 18\
%     \___/ 6 \___/ 2 \___/
%     / 11\___/ 7 \___/ 19\
%     \___/ 10\___/ 8 \___/
%         \___/ 9 \___/
%             \___/
BS_location = [0, 0;...                                         % cell 1
    
              (3/2)*cell_radius,   -(sqrt(3)/2)*cell_radius;... % cell 2-7
              (3/2)*cell_radius,   (sqrt(3)/2)*cell_radius;...
               0,                  sqrt(3)*cell_radius;...
               -(3/2)*cell_radius, (sqrt(3)/2)*cell_radius;...
               -(3/2)*cell_radius, -(sqrt(3)/2)*cell_radius;...
               0,                  -sqrt(3)*cell_radius;...
               
               (3/2)*cell_radius,  -sqrt(3)*(3/2)*cell_radius;... % cell 8-19
               0,                  -sqrt(3)*2*cell_radius;...
               -(3/2)*cell_radius, -sqrt(3)*(3/2)*cell_radius;...
               -3*cell_radius,     -sqrt(3)*cell_radius;...
               -3*cell_radius,     0;...
               -3*cell_radius,     sqrt(3)*cell_radius;...
               -(3/2)*cell_radius, sqrt(3)*(3/2)*cell_radius;...
               0,                  sqrt(3)*2*cell_radius;...
               (3/2)*cell_radius,  sqrt(3)*(3/2)*cell_radius;...
               3*cell_radius,      sqrt(3)*cell_radius;...
               3*cell_radius,      0;...
               3*cell_radius,      -sqrt(3)*cell_radius];
           
BS_location = BS_location(1:N,:); %Take out the N-cells we want. 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Generate users location, uniform over each cell. %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   y ^
%     |
%   R |________                         _______
%     | 2 \  1 |                       /1f | 2 \
%   0 |____\___|____> x     ==>       /____|____\   f = flipped vertically
%     | 4  / 3 |R                     \ 3f | 4  /
%  -R |___/____|                       \___|___/
%     |
x = cell_radius * rand(N,U);                % Create x uniform in (0,R).
y = (rand(N,U)-0.5)*2*cell_radius;          % Create y uniform in (-R,R).

%Split the square (0,0 -> R,R) into two similar right trapezoids.
%Use logicals to avoid for loop.
IND = abs(y) + 3*x > 2*cell_radius; 
x(IND) = x(IND) - cell_radius;
y(IND) = sign(y(IND))*cell_radius - y(IND);

%Scale to fit the hexagon(s).
x = x * (3/2);
y = y * sqrt(3)/2;

%Exclude disk in the center.
while any(any(sqrt(x.^2 + y.^2) < exclude_disk_radius))
    IND = sqrt(x.^2 + y.^2) < exclude_disk_radius;
    ssIND= sum(sum(IND));
    newXvals = zeros(1,ssIND);
    newYvals = zeros(1,ssIND);
    %Re-draw coordinates
    newXvals = (cell_radius * rand(ssIND,1));           % uniform rand.dist
    newYvals = (rand(ssIND,1)-0.5)*2*cell_radius;

    IND2 = abs(newYvals) + 3*newXvals > 2*cell_radius;  % flip
    newXvals(IND2) = newXvals(IND2) - cell_radius;      
    newYvals(IND2) = sign(newYvals(IND2))*cell_radius - newYvals(IND2);
    %Scale.
    newXvals = newXvals * 3/2;
    newYvals = newYvals * sqrt(3)/2;
    %Substitute
    x(IND) = newXvals; 
    y(IND) = newYvals;
end

%Locate in cells, i.e., Array NxUx2.
user_loc_cell(:,:,1) = repmat(BS_location(:,1),1,U) + x;  %x coordinate
user_loc_cell(:,:,2) = repmat(BS_location(:,2),1,U) + y;  %y coordinate


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Distance from user u in cell n to BS in cell "all" n.  %
% dist_user_BS_cell : N x U x N array                    %
% N - cell index. U - number of users in each cell.      %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

for n=1:N
    dist_user_BS_cell(1:N,1:U,n) = sqrt( (user_loc_cell(:,:,1) - repmat(BS_location(n,1),N,U) ).^2 ...
                                        +(user_loc_cell(:,:,2) - repmat(BS_location(n,2),N,U) ).^2 );
end

% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% % Scatter plot of user location.  %
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% figure(1);%fig);
% hold on;
% for n=1:N
%     scatter(user_loc_cell(n,:,1), user_loc_cell(n,:,2));
% end
% title('Scatter plot of user location.');
% xlabel('x (m)');
% ylabel('y (m)');

return;