%Calculate path loss according to the COST231_Hata model.
%In addition, Rayleigh fading and shadowing are added.
%Finally, thermal noise is calculated.
%
%[G_lin sig] = COST231_Hata(f_c, terminal_height, BS_height, Grx, Gtx,
%shadow_dev,N,U,BS_location, user_loc_cell, dist_user_BS_cell), where:
%
%G_lin is the channel power gain matrix (in linear scale) with dimensions
%N x N x U, i.e., for user 1<u<U in each cell, there is an NxN matrix.
%
%sig is a 1xN thermal noise vector.
%
%f_c                    % carrier frequency in MHz
%BS_height              % BS height in meters
%terminal_height        % terminal height in meters
%Gtx                    % Tx antenna gain in dB
%Grx                    % Rx antenna gain in dB
%shadow_dev             % st.dev of shadowing in dB
%N                      % number of cells. 1<=N<=7
%U                      % number of users per cell
%
%BS_location, user_loc_cell, and dist_user_BS_cell are locations of users
%and BSs as calculated by BSUserLocation.m.

function [G_lin sig] = COST231_Hata(f_c, terminal_height, BS_height, Grx, Gtx, shadow_dev,N,U,BS_location, user_loc_cell, dist_user_BS_cell)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   Script to calculate propagation.        %
%   Author: Anders Gjendemsj, IET, NTNU.   %
%   Email: gjendems@iet.ntnu.no             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%
% Defining variables.    %
%%%%%%%%%%%%%%%%%%%%%%%%%%
a_terminal_small_med =(1.1 * log10(f_c) - .7)*terminal_height - (1.56*log10(f_c) - .8); % in dB, small, med cities.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Channel power gains.                                %
% G_lin / G_dB :  N x N x U  array                    %
% N - cell index. U - number of users in each cell.   %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
G_lin = zeros(N,N,U);
G_dB = zeros(N,N,U);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Channel power gains.                                %
% G_lin / G_dB :  N x N x U  array                    %
% N - cell index. U - number of users in each cell.   %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

for u=1:U
    G_dB(1:N,1:N,u) = - propagation_model( dist_user_BS_cell(1:N,u,1:N),a_terminal_small_med) + Grx + Gtx; %Wanted communication links.
end
 
% %Add Rayleigh fading, powers will be exp. dist, add shadowing.

%Alternative with statistics toolbox.
%G_dB = 10*log10( exprnd( 10.^(G_dB./10) ) ) + normrnd(0,shadow_dev, N, N,U);

% Alternative without statistics toolbox.
G_dB = 10*log10( -10.^(G_dB./10) .* log(rand(size(G_dB))) ) + randn( size(G_dB) ) .* shadow_dev;

%Convert to linear scale.
G_lin = 10.^(G_dB./10);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Calculate noise levels, sigma_n             %
% Currently, all noises are equal.            %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
boltz_const = 1.3806503*10^(-23);
T_0 = 290;                                  % ambient temperature in kelvins
B = 1*10^(6);                               % equivalent bandwidth 1MHz
therm_noise_W = boltz_const*T_0*B;          % thermal noise power in W
sig = ones(1,N)*therm_noise_W;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Calculate loss in dB, according to COST-231 Walfisch model.      %
%See pp. 134-135 in "Digital Mobile Radio towards future          %
%generation systems".                                             %
%Available at http://www.iet.ntnu.no/~gjendems/documents          %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

        
        %Cm = 0; % for medium sized cities and suburbs, 3 dB for metropolitian areas.
function path_loss = propagation_model(distance_user_bs, a_terminal_small_med)
        %a_terminal = 3.2 * (log10(11.75*terminal_height))^2 - 4.97; %in dB, big cities.
        path_loss = 46.3 + 33.9 * log10(f_c) - 13.82 * log10(BS_height) - a_terminal_small_med ...
           + (44.9-6.55*log10(BS_height))*log10(distance_user_bs/1000);% + Cm=0;
end


end