% Program to calculate:
%   - path loss (COST231 Walfish model)
%   - includes shadowing (not fading)
%   - thermal noise
%
% [G_lin sig] = COST231_Walfisch_LOS(f_c, Grx, Gtx,
%                             shadow_dev, N, U, BS_location, user_loc_cell,
%                             dist_user_BS_cell)
%
%G_lin is the channel power gain matrix (in linear scale) with dimensions
%N x N x U, i.e., for user 1<u<U in each cell, there is an NxN matrix.
%
%sig is a 1xN thermal noise vector
%
%f_c                    % carrier frequency in MHz
%shadow_dev             % st.dev of shadowing [dB]
%Grx, Gtx               % antenna gain of receiver, transceiver [dB]
%N                      % number of cells
%U                      % number of users per cell
%
%BS_location, user_loc_cell, and dist_user_BS_cell are locations of users
%and BSs as calculated by BSUserLocation.m.

function [G_lin, sig] = COST231_Walfisch_LOS(f_c, Grx, Gtx, shadow_dev, N, ...
                                             U, BS_location, user_loc_cell, dist_user_BS_cell, K)


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  Script to calculate propagation.                   %
%  Author: Anders Gjendemsj, IET, NTNU.              %
%  Email: gjendems@iet.ntnu.no                        %
%                                                     %
%  Modified: Eivind Syvertsen <eivindsy@pvv.org>      %
%   -removed inconsistent fading, added antenna gains %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Channel power gains.                                %
% G_lin , G_dB :  N x N x U  array                    %
% N - cell index. U - number of users in each cell.   %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%G_lin = zeros(N,N,U);
G_dB = zeros(N,N,U);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Channel power gains.                                %
% G_lin , G_dB :  N x N x U  array                    %
% N - cell index. U - number of users in each cell.   %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

for u=1:U
    G_dB(1:N,1:N,u) = - propagation_model( dist_user_BS_cell(1:N,u,1:N), f_c);
    
    % Calculating parameters for Rician multipath fading
    s_ii = mean(10.^(diag(G_dB(:,:,u))./10)) ./ (K+1);
    s_ij = mean(mean(10.^( spdiags(spdiags(G_dB(:,:,u) - diag(diag(G_dB(:,:,u)))))./10))) ./ (K+1);
    s(:,:,u) = diag((s_ii-s_ij)*ones(1,N)) + s_ij;
    
end
 

%Alternative with statistics toolbox.
%G_dB = 10*log10( exprnd( 10.^(G_dB./10) ) ) + normrnd(0,shadow_dev, N,
%   N,U) + Grx + Gtx;

% Alternative without statistics toolbox.
% G_dB = 10*log10( -10.^(G_dB./10) .* log(rand(size(G_dB))) ) + randn( ...
%     size(G_dB) ) .* shadow_dev + Grx + Gtx;

% Make channel characteristics Rician and add shadowing
% s_ii = mean(10.^(diag(G_dB(:,:,1))./10)) ./ (K+1);
% s_ij = mean(mean(10.^( spdiags(spdiags(G_dB(:,:,1) - diag(diag(G_dB(:,:,1)))))./10))) ./ (K+1);
% s = diag((s_ii-s_ij)*ones(1,N)) + s_ij;
s

G_dB = 10*log10( ricernd( 10.^(G_dB./10), s )) + randn( size(G_dB) ) .* shadow_dev + Grx + Gtx;

%Convert to linear scale.
G_lin = 10.^(G_dB./10);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Calculate noise levels, sigma_n             %
% Currently, all noises are equal.            %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
boltz_const = 1.3806503*10^(-23);
T_0 = 290;                                  % ambient temperature in kelvins
B = 1*10^(6);                               % equivalent bandwidth 1MHz
therm_noise_W = boltz_const*T_0*B;          % thermal noise power in W
sig = ones(1,N)*therm_noise_W;



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Calculate loss in dB, according to COST-231 Walfisch model.      %
% See pp. 135-139 in "Digital Mobile Radio towards future          %
% generation systems".                                             %
% Available at http://www.iet.ntnu.no/~gjendems/documents          %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        
function path_loss = propagation_model(distance_user_bs, f_c)
        path_loss = 42.6 + 26*log10(distance_user_bs/1000) + 20*log10(f_c);
