%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Simulation script for N hexagonal cells.    %
% Comparement of total network capacity for   %
%  -optimal power allocation (solved with GP) %
%  -binary power allocation                   %
%  -full power allocation                     %
% for an average user pair case.              %
%                                             %
% Author: Anders Gjendemsj???                   %
% Modified: Eivind Syvertsen (Sept 2006)      %
%           Sigbj?rn Dybdahl (March 2007)     %
%   -removed GP solution                      %
%   -added support for p_min>0                % 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clc; 
clear all;
%cd ~/project/matlab/Ncells/N7/

%path(path, '../');          % path to BSUserLocation() and COST231_Walfisch_LOS()
%path(path, '../../ggplab'); % path to gpposy() (geometric optimization)
rand('state',2600);         % Set initial seed for random functions
randn('state',2600);
tic;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Defining simulation parameters %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
number_simulations = 10000;  %2000;
%maxGPiter = 40;

N = 2;                      % number of cells.
U = 1;                      % #users per cell
P_max = 1;                  % maximum transmit power [W]
P_min1 = 0.1;               % minimum transmit power levels [W]
P_min2 = 0.2;
P_min3 = 0.3;

% Path loss model parameters
cell_radius = 1000;         % cell radius [m]
f_c = 1800;                 % carrier frequency [MHz]
exclude_disk_radius = 20;   % exclude a disc with this radius [m] around BS
                            % due to limitations in path loss model
                            % (e.g. COST 231)
terminal_height = 1;         % terminal height in meters
BS_height = 30;             % Bs height in meters
shadow_dev = 10;            % st.dev of shadowing [dB]
Gtx = 16;                   % transceiver antenna gain [dB]
Grx = 6;                    % receiver antenna gain [dB]
K = 5;                      % Rician K factor

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Internal variables.          %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% GP   : GP optimized power levels, P \in C[0,P_max]
% BINx : binary power levels, P \in {P_minx, P_max}
% FULL : full power, P = P_max
%capacityGP = zeros(1,number_simulations);  % capacity per simulation
capacityBIN0 = zeros(1,number_simulations); 
capacityBIN1 = zeros(1,number_simulations);   
capacityBIN2 = zeros(1,number_simulations);   
capacityBIN3 = zeros(1,number_simulations);   
capacityFULL = zeros(1,number_simulations);  

%sum_capacityGP = 0; %zeros(1,N(n));  % sum capacity
sum_capacityBIN0 = 0;
sum_capacityBIN1 = 0;
sum_capacityBIN2 = 0;
sum_capacityBIN3 = 0;
sum_capacityFULL = 0;

%powerAllocGP = zeros(number_simulations,N);  % power allocation for each cell
powerAllocBIN0 = zeros(number_simulations,N);
powerAllocBIN1 = zeros(number_simulations,N);
powerAllocBIN2 = zeros(number_simulations,N);
powerAllocBIN3 = zeros(number_simulations,N);
%powerAlocBIN_N7 = ones(number_simulations,N);
%Termination criterion for GP optimizing
epsStop = 10^(-4);

% Removed: Only one cell structure suppported.
%for n=1:numel(N)    % for each cell structure
    
disp(' ');
disp(' ');
disp('%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%');
disp('%Starting simulation with parameters:%');
disp('%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%');
disp(['# simulations: ', num2str(number_simulations)]);
disp(['# cells: ', num2str(N)]);
disp(['# users per cell: ', num2str(U)]);
disp(['P_min between: ', num2str(0), ' and ', num2str(P_min3)]);
disp(['P_max: ', num2str(P_max)]);
disp(['Cell wrapping: no']);

% create binary power values (list of all permutation)
P = (dec2bin(1:(2^N - 1)) -'0')*P_max;
P1 = (dec2bin(1:(2^N - 1)) -'0')*(P_max-P_min1)+P_min1;
P2 = (dec2bin(1:(2^N - 1)) -'0')*(P_max-P_min2)+P_min2;
P3 = (dec2bin(1:(2^N - 1)) -'0')*(P_max-P_min3)+P_min3;

                                          
sig = zeros(1,N);                          % sigma_i per cell
upperBound = ones(N,1)*P_max;              % power constraint, upper bound

%btmp = zeros(3425,1);                      % b vector for GP optimization
%load Atmp Atmp;                            % load temporary/initial A matrix

BS_location = zeros(N,2);                  % (x,y) coordinates for each BS
user_loc_cell = zeros(N,U,2);              % (x,y) coordinates for each user
dist_user_BS_cell = zeros(N,U,N);          % distance from user u in cell n to the N other BS

G = zeros(N,N,number_simulations);         % channel gain matrix, per simulation


% Monte Carlo simulation loop
for i = 1:number_simulations
    
    %disp(['Iteration no.: ' num2str(i)]);
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % G = [ G_11 G_12 ... G_1N                          %
    %       G_21 G_22 ... G_2N                          %
    %       ..................                          %
    %       G_N1 ........ G_NN ],                       %
    %   where G_ni is the gain to the user in cell n,   %
    %   from the BS in cell i.                          %
    % sig = [sigma_1^2 sigma_2^2 ... sigma_N^2];        %
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    [BS_location, user_loc_cell, dist_user_BS_cell] = BSUserLocation(N, U, cell_radius, exclude_disk_radius);
    % Using the COST 231 Walfisch LOS-model
    [G(:,:,i), sig] = COST231_Walfisch_LOS(f_c, Grx, Gtx, shadow_dev, N, U, BS_location, user_loc_cell, dist_user_BS_cell, K);
    % Using the COST 231 extension to Hata model
    % [G(:,:,i), sig] = COST231_Hata(f_c, terminal_height, BS_height, Grx, Gtx, shadow_dev,N,U,BS_location, user_loc_cell, dist_user_BS_cell);

    sig = sig';

    
    % Initialization vectors for GP iterations
    %Pold = P_max*rand(N,1);
%     Pold = ones(N,1);
%     btmp = [b1(G(:,:,i),sig(1)); b2(G(:,:,i),sig(1)); b3(G(:,:,i),sig(1)); b4(G(:,:,i),sig(1)); b5(G(:,:,i),sig(1)); b6(G(:,:,i),sig(1))];
% 
%     % GP iteration loop
%     for k = 1:maxGPiter
%        
% 
%         %Step 1. Evaluate the denominator posynomial of the objective
%         %function with the given E. In this version, split up to
%         %increase speed.
%         denomEval = G(:,:,i) * Pold + sig; %create [I,II,III]' (see notes).
% 
%         
%         %Step 2. Compute for each term in denomPos (consists of each row in G), 
%         %alfa_l = value of l'term / value of posynomial.
%         alfa = [(G(:,:,i).*repmat(Pold',N,1)) sig] ./ repmat(denomEval,1,N+1);
% 
%         
%         %Step 3. Condense the den. posynomials.
%             %Step 3-1 calculate constant factor
%             constFactor = prod( (sig./alfa(:,N+1)).^(alfa(:,N+1)) ) * ... 
%                           prod ( prod ( (G(:,:,i)./alfa(:,1:N)).^(alfa(:,1:N)) ) );
%             %Step 3-2, get param from f_param
%             A = Atmp - repmat(sum(alfa(:,1:N)),length(Atmp),1);
% 
%         %Step 4. Solve GP.
%         %Using gpposy function from GGPLAB toolbox, feeding it with posynomials.
%         [E,status,lambda,nu] = gpposy (A, btmp / constFactor, length(btmp),[],[],[],upperBound,1);
%         
% 
%         %Check for convergence of solution.
% 	eps = norm(E-Pold);
% 
%         if eps < epsStop
%            %disp(['Norm less than eps, we stop. Iteration: ' num2str(k) ]);
%            break
%         end
% 
%         %else, we continue with more GP iterations
%         Pold = E;
% 
%     end %end GP iterations, solution stored in E.
% 
%     
%     
%     
%     disp(['Monte Carlo iteration no.: ' int2str(i) ' (' int2str(k) ' GP iterations required [eps=' num2str(eps) '])']);
%      
    %Split channel gain matrix G into desired channels vs interfering channels.
    g_nn = diag(G(:,:,i));                     % Diagonal elements only
                                               % (wanted transmission
                                               % channel gains only)
    G_interference = G(:,:,i) - diag(g_nn);    % Non-diagonal elements
                                               % only (interfering
                                               % transmission channel
                                               % gains only)
% 
%     %Calculate GP capacity
%     f1 = E .* g_nn;                            % SNR nominator; wanted signal
%     f2 = (G_interference * E) + sig;           % SNR Denominator;
%                                                % interference + noise power
%     factor = 1 + f1./f2;
%     tmp_valueGP = log2(prod(factor));          % Shannon: C =
%                                                % SUM_j(log2(1+SNRj)) =
%                                                % log2(PRODUCT_j(1+SNRj))
    

    %Calculate BINARY capacity
    %G_nn = zeros(2^N-1,N); %not necessary
    G_nn = (g_nn(:,ones(2^N-1,1)))';        % Tony's trick.
    sig = repmat(sig,1,2^N-1);
    
    %For (a) given row(s) in P.
    f1 = P .* G_nn;                      % Wanted transmission, %G_nn replicated diagonal elements of G
    f2 = (G_interference * P')' + sig';  % Denominator; G_interference = G with diag. elements = 0.
    factor = 1 + f1./f2;
    capValsAllP = log2( prod(factor,2) );
    [capacityBIN0(i),indexBIN0] = max(capValsAllP);   % Select the best from
                                                    % all available binary
                                                    % power possibilities

    %Calculate FULL capacity
    capacityFULL(i) = capValsAllP(2^N-1);   % Use the last line in 
                                            % capCalsAllP (where all 
                                            % P_i = P_max)
                                            
    %Calculate capacity for P_min>0
    % P_min1
    f1 = P1 .* G_nn;                      % Wanted transmission, %G_nn replicated diagonal elements of G
    f2 = (G_interference * P1')' + sig';  % Denominator; G_interference = G with diag. elements = 0.
    factor = 1 + f1./f2;
    capValsAllP = log2( prod(factor,2) );
    [capacityBIN1(i),indexBIN1] = max(capValsAllP); 
    
    % P_min2
    f1 = P2 .* G_nn;                      % Wanted transmission, %G_nn replicated diagonal elements of G
    f2 = (G_interference * P2')' + sig';  % Denominator; G_interference = G with diag. elements = 0.
    factor = 1 + f1./f2;
    capValsAllP = log2( prod(factor,2) );
    [capacityBIN2(i),indexBIN2] = max(capValsAllP); 
    
    % P_min3
    f1 = P3 .* G_nn;                      % Wanted transmission, %G_nn replicated diagonal elements of G
    f2 = (G_interference * P3')' + sig';  % Denominator; G_interference = G with diag. elements = 0.
    factor = 1 + f1./f2;
    capValsAllP = log2( prod(factor,2) );
    [capacityBIN3(i),indexBIN3] = max(capValsAllP); 
    
    
    %Keep track of solutions, accumulate capacity values over all Monte
    %Carlo simulations
    
    %Binary
    sum_capacityBIN0 = sum_capacityBIN0 + capacityBIN0(i);
    sum_capacityBIN1 = sum_capacityBIN1 + capacityBIN1(i);
    sum_capacityBIN2 = sum_capacityBIN2 + capacityBIN2(i);
    sum_capacityBIN3 = sum_capacityBIN3 + capacityBIN3(i);

    %Full       
    sum_capacityFULL = sum_capacityFULL + capacityFULL(i);
    
%     %GP
%     % Because of numerical round-offs with the GP optimization, the binary
%     % power case might provide higher capacity: in this case, use the
%     % binary power capacity as GP capacity (optimal capacity).
%     [capacityGP(i), indexHeur] = max([capacityBIN(i), tmp_valueGP]);
%     sum_capacityGP = sum_capacityGP + capacityGP(i);
%     
%     if indexHeur == 2
%        powerAllocGP_N7(i,:) = E';             % GP better than BIN
%     else
%        powerAllocGP_N7(i,:) = P(indexBIN,:);   % BIN better than GP => Use
%                                               % BIN power as GP power
%                                               % allocation
%     end

    powerAllocBIN0(i,:) = P(indexBIN0,:);
    powerAllocBIN1(i,:) = P1(indexBIN1,:);
    powerAllocBIN2(i,:) = P2(indexBIN2,:);
    powerAllocBIN3(i,:) = P3(indexBIN3,:);

end %for loop, number of Monte Carlo simulations



% %%%%%%%%%%%%%%%%%%%%%%
% % Preparing results. %
% %%%%%%%%%%%%%%%%%%%%%%
avg_syst_capBIN0 = sum_capacityBIN0/number_simulations;
avg_syst_capBIN1 = sum_capacityBIN1/number_simulations;
avg_syst_capBIN2 = sum_capacityBIN2/number_simulations;
avg_syst_capBIN3 = sum_capacityBIN3/number_simulations;
avg_syst_capFULL = sum_capacityFULL/number_simulations;

avg_per_cell_capBIN0 = avg_syst_capBIN0/N;
avg_per_cell_capBIN1 = avg_syst_capBIN1/N;
avg_per_cell_capBIN2 = avg_syst_capBIN2/N;
avg_per_cell_capBIN3 = avg_syst_capBIN3/N;
avg_per_cell_capFULL = avg_syst_capFULL/N;

disp(' ');disp(' ');
disp('%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%');
disp(['%       Simulation finished, N=', num2str(N),'         %']);
disp('%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%');
disp(['Avg. system capacity (per cell), pow.ad BIN0: ', num2str(avg_per_cell_capBIN0)]);
disp(['Avg. system capacity (per cell), pow.ad BIN1: ', num2str(avg_per_cell_capBIN1)]);
disp(['Avg. system capacity (per cell), pow.ad BIN2: ', num2str(avg_per_cell_capBIN2)]);
disp(['Avg. system capacity (per cell), pow.ad BIN3: ', num2str(avg_per_cell_capBIN3)]);
disp(['Avg. system capacity (per cell), pow.ad FULL: ', num2str(avg_per_cell_capFULL)]);

%Store results

sti='simulation_results/N2/';

save([sti, 'sum_capacityBIN0'], 'sum_capacityBIN0');
save([sti, 'sum_capacityBIN1'], 'sum_capacityBIN1');
save([sti, 'sum_capacityBIN2'], 'sum_capacityBIN2');
save([sti, 'sum_capacityBIN3'], 'sum_capacityBIN3');
save([sti, 'sum_capacityFULL'], 'sum_capacityFULL');

save([sti, 'powerAllocBIN0'], 'powerAllocBIN0');
save([sti, 'powerAllocBIN1'], 'powerAllocBIN1');
save([sti, 'powerAllocBIN2'], 'powerAllocBIN2');
save([sti, 'powerAllocBIN3'], 'powerAllocBIN3');

save([sti, 'G'], 'G');

save([sti, 'capacityBIN0'], 'capacityBIN0');
save([sti, 'capacityBIN1'], 'capacityBIN1');
save([sti, 'capacityBIN2'], 'capacityBIN2');
save([sti, 'capacityBIN3'], 'capacityBIN3');
save([sti, 'capacityFULL'], 'capacityFULL');

save([sti, 'number_simulations'], 'number_simulations');

%end %for loop, number of cells.

toc





 
