%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Simulation script for N hexagonal cells.    %
% Comparement of total network capacity for   %
%  -optimal power allocation (solved with GP) %
%  -binary power allocation                   %
%  -full power allocation                     %
% for an average user pair case.              %
%                                             %
% Author: Anders Gjendemsj???                   %
% Modified: Eivind Syvertsen (Sept 2006)      %
%           Sigbj?rn Dybdahl (March 2007)     %
%   -added proportional fair scheduling       %
%   -removed GP solution                      %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clc; 
clear all;
close all;
%cd ~/project/matlab/Ncells/N7/

%path(path, '../');          % path to BSUserLocation() and COST231_Walfisch_LOS()
%path(path, '../../ggplab'); % path to gpposy() (geometric optimization)
rand('state',2600);         % Set initial seed for random functions
randn('state',2600);
tic;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Defining simulation parameters %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
number_simulations = 10;  %2000;
%maxGPiter = 40;

N = 7;                      % number of cells.
U = 3;                      % #users per cell
P_max = 1;                  % maximum transmit power [W]

ic = 100;                   % time horizon for PFS

% Path loss model parameters
cell_radius = 1000;         % cell radius [m]
f_c = 1800;                 % carrier frequency [MHz]
exclude_disk_radius = 20;   % exclude a disc with this radius [m] around BS
                            % due to limitations in path loss model
                            % (e.g. COST 231)
terminal_height = 1;         % terminal height in meters
BS_height = 30;             % Bs height in meters
shadow_dev = 10;            % st.dev of shadowing [dB]
Gtx = 16;                   % transceiver antenna gain [dB]
Grx = 6;                    % receiver antenna gain [dB]
K = 5;                      % Rician K factor

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Internal variables.          %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% BIN : binary power levels, P \in {0, P_max}
% FULL : full power, P = P_max
capacityBIN = zeros(1,number_simulations);
capacityBINPFS = zeros(1,number_simulations);
capacityFULL = zeros(1,number_simulations);  

%sum_capacityGP = 0; %zeros(1,N(n));  % sum capacity
sum_capacityBIN = 0;
sum_capacityBINPFS = 0;
sum_capacityFULL = 0;

%powerAllocGP = zeros(number_simulations,N);  % power allocation for each cell
powerAllocBIN = zeros(number_simulations,N);
powerAllocBINPFS = zeros(number_simulations,N);

%Termination criterion for GP optimizing
%epsStop = 10^(-4);

% Removed: Only one cell structure suppported.
%for n=1:numel(N)    % for each cell structure
    
disp(' ');
disp(' ');
disp('%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%');
disp('%Starting simulation with parameters:%');
disp('%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%');
disp(['# simulations: ', num2str(number_simulations)]);
disp(['# cells: ', num2str(N)]);
disp(['# users per cell: ', num2str(U)]);
disp(['P_max: ', num2str(P_max)]);
disp(['ic: ', num2str(ic)]);

% create binary power values (list of all permutation)
P = (dec2bin(1:(2^N - 1)) -'0')*P_max;
                                          
sig = zeros(1,N);                          % sigma_i per cell
upperBound = ones(N,1)*P_max;              % power constraint, upper bound

%btmp = zeros(3425,1);                      % b vector for GP optimization
%load Atmp Atmp;                            % load temporary/initial A matrix

BS_location = zeros(N,2);                  % (x,y) coordinates for each BS
user_loc_cell = zeros(N,U,2);              % (x,y) coordinates for each user
dist_user_BS_cell = zeros(N,U,N);          % distance from user u in cell n to the N other BS

%G = zeros(N,N,number_simulations);         % channel gain matrix, per simulation
G = zeros(N,N,U);

T = ones(1,N*U);                           % average throughput vector


% Monte Carlo simulation loop
for i = 1:number_simulations
    
    %disp(['Iteration no.: ' num2str(i)]);
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % G = [ G_11 G_12 ... G_1N                          %
    %       G_21 G_22 ... G_2N                          %
    %       ..................                          %
    %       G_N1 ........ G_NN ],                       %
    %   where G_ni is the gain to the user in cell n,   %
    %   from the BS in cell i.                          %
    % sig = [sigma_1^2 sigma_2^2 ... sigma_N^2];        %
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    [BS_location, user_loc_cell, dist_user_BS_cell] = BSUserLocation(N, U, cell_radius, exclude_disk_radius);
    % Using the COST 231 Walfisch LOS-model
    [G(:,:,:), sig] = COST231_Walfisch_LOS(f_c, Grx, Gtx, shadow_dev, N, U, BS_location, user_loc_cell, dist_user_BS_cell, K);
    % Using the COST 231 extension to Hata model
    % [G(:,:,i), sig] = COST231_Hata(f_c, terminal_height, BS_height, Grx, Gtx, shadow_dev,N,U,BS_location, user_loc_cell, dist_user_BS_cell);

    sig = sig';

    for u = 1:U
  
        %Split channel gain matrix G into desired channels vs interfering channels.
        g_nn = diag(G(:,:,u));                     % Diagonal elements only
                                                   % (wanted transmission
                                                   % channel gains only)
        G_interference = G(:,:,u) - diag(g_nn);    % Non-diagonal elements
                                                   % only (interfering
                                                   % transmission channel
                                                   % gains only)
    % 
    %     %Calculate GP capacity
    %     f1 = E .* g_nn;                            % SNR nominator; wanted signal
    %     f2 = (G_interference * E) + sig;           % SNR Denominator;
    %                                                % interference + noise power
    %     factor = 1 + f1./f2;
    %     tmp_valueGP = log2(prod(factor));          % Shannon: C =
    %                                                % SUM_j(log2(1+SNRj)) =
    %                                                % log2(PRODUCT_j(1+SNRj))


        %Calculate BINARY capacity
        %G_nn = zeros(2^N-1,N); %not necessary
        G_nn = (g_nn(:,ones(2^N-1,1)))';        % Tony's trick.
        sig = repmat(sig,1,2^N-1);

        %For (a) given row(s) in P.
        f1 = P .* G_nn;                      % Wanted transmission, %G_nn replicated diagonal elements of G
        f2 = (G_interference * P')' + sig';  % Denominator; G_interference = G with diag. elements = 0.
        factor = 1 + f1./f2;
        capValsAllP = log2( prod(factor,2) );

        [tempCapacityBIN(u),tempIndexBIN(u)] = max(capValsAllP);   % Select the best from
                                                        % all available binary
                                                        % power possibilities

        %Perform Proportional Fair Scheduling calculations
        %1. Find cell k* = argmax R_k[i]/T_k[i]
        
               
        [value1, allocPos] = max(log2(factor));    
        [value2, cell] = max(value1./T);
        indexBINPFS=allocPos(cell);

        %2. Update T_k[i+1]
        T = (1-1/ic)*T;
        T(cell) = T(cell) + (1/ic)*value1(cell); 

        %3. Calculate BINARY-PFS capacity
        tempCapacityBINPFS(u) = capValsAllP(indexBINPFS);

        %Calculate FULL capacity
        tempCapacityFULL(u) = capValsAllP(2^N-1);   % Use the last line in 
                                                % capCalsAllP (where all 
                                                % P_i = P_max)
    end
    
    [capacityBIN(i), indexBIN] = max(tempCapacityBIN);
    capacaityFULL(i) = tempCapacityFULL(indexBIN);
                                            
    
    
    %Keep track of solutions, accumulate capacity values over all Monte
    %Carlo simulations
    
    %Binary
    sum_capacityBIN = sum_capacityBIN + capacityBIN(i);
    sum_capacityBINPFS = sum_capacityBINPFS + capacityBINPFS(i);

    %Full       
    sum_capacityFULL = sum_capacityFULL + capacityFULL(i);
    

    powerAllocBIN(i,:) = P(tempIndexBIN(indexBIN),:);
    powerAllocBINPFS(i,:) = P(indexBINPFS,:);

end %for loop, number of Monte Carlo simulations



% %%%%%%%%%%%%%%%%%%%%%%
% % Preparing results. %
% %%%%%%%%%%%%%%%%%%%%%%
avg_syst_capBIN = sum_capacityBIN/number_simulations;
avg_syst_capBINPFS = sum_capacityBINPFS/number_simulations;
avg_syst_capFULL = sum_capacityFULL/number_simulations;

avg_per_cell_capBIN = avg_syst_capBIN/N;
avg_per_cell_capBINPFS = avg_syst_capBINPFS/N;
avg_per_cell_capFULL = avg_syst_capFULL/N;

disp(' ');disp(' ');
disp('%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%');
disp(['%       Simulation finished, N=', num2str(N),'         %']);
disp('%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%');
disp(['Avg. system capacity (per cell), pow.ad BIN: ', num2str(avg_per_cell_capBIN)]);
disp(['Avg. system capacity (per cell), pow.ad BINPFS: ', num2str(avg_per_cell_capBINPFS)]);
disp(['Avg. system capacity (per cell), pow.ad FULL: ', num2str(avg_per_cell_capFULL)]);

%Store results

% sti='simulation_results/pfs/';
% 
% save([sti, 'sum_capacityBIN'], 'sum_capacityBIN');
% save([sti, 'sum_capacityBINPFS'], 'sum_capacityBINPFS');
% save([sti, 'sum_capacityFULL'], 'sum_capacityFULL');
% 
% save([sti, 'powerAllocBIN'], 'powerAllocBIN');
% save([sti, 'powerAllocBINPFS'], 'powerAllocBINPFS');
% 
% save([sti, 'G'], 'G');
% 
% save([sti, 'capacityBIN'], 'capacityBIN');
% save([sti, 'capacityBINPFS'], 'capacityBINPFS');
% save([sti, 'capacityFULL'], 'capacityFULL');
% 
% save([sti, 'number_simulations'], 'number_simulations');

%end %for loop, number of cells.

toc
